import setuptools

setuptools.setup(
    name="Media Manipulation Detector",
    version="0.0.1",
    author="David Maia",
    description="Module created to detect manipulated media",
    packages=["MediaManipulationDetector"]
)
